@@ -4,7 +4,7 @@ use super::{BoundedLattice, Lattice};
 use std::cmp::Ordering;
 
 #[derive(Clone, Copy, PartialEq, Eq, Debug)]
-/// A wrapper for tuple types that provides an implementation of `PartialOrd` using
+/// A wrapper for tuple types and arrays that implements `PartialOrd` using
 /// [product-order](https://en.wikipedia.org/wiki/Product_order) semantics. 
 /// 
 /// `Lattice` and `BoundedLattice` traits are also implemented.
@@ -97,3 +97,77 @@ fn test_product_lattice(){
    assert!(!{Product((1,4)) < Product((2,3))});
    assert!(Product((1,4)).partial_cmp(&Product((2,3))) == None);
 }
+
+
+impl <const N: usize, T: PartialOrd> PartialOrd for Product<[T; N]> {
+   fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+      let mut ord = Ordering::Equal;
+      for i in 0..N {
+         let ith_ord = self.0[i].partial_cmp(&other.0[i]);
+         match ith_ord {
+            None => return None,
+            Some(ith_ord) => match combine_orderings(ith_ord, ord) {
+               Some(new_ord) => ord = new_ord,
+               None => return None,
+            },
+         }
+      }
+      Some(ord)
+   }
+}
+
+#[test]
+fn test_product_of_array_partial_ord() {
+   let a1 = Product([1, 2, 3]);
+   let a2 = Product([1, 3, 4]);
+   assert_eq!(a1.partial_cmp(&a2), Some(Ordering::Less));
+   assert_eq!(a2.partial_cmp(&a1), Some(Ordering::Greater));
+
+   let a3 = Product([0, 2, 4]);
+   assert_eq!(a1.partial_cmp(&a3), None);
+   assert_eq!(a3.partial_cmp(&a1), None);
+
+   assert_eq!(a2.partial_cmp(&a3), Some(Ordering::Greater));
+   assert_eq!(a3.partial_cmp(&a2), Some(Ordering::Less));
+}
+
+impl <const N: usize, T: Lattice> Lattice for Product<[T; N]> {
+   fn meet(mut self, other: Self) -> Self {
+      for (l,r) in self.0.iter_mut().zip(other.0){
+         l.meet_mut(r);
+      }
+      self
+   }
+
+   fn join(mut self, other: Self) -> Self {
+      for (l,r) in self.0.iter_mut().zip(other.0){
+         l.join_mut(r);
+      }
+      self
+   }
+}
+
+impl <const N: usize, T: BoundedLattice> BoundedLattice for Product<[T; N]> {
+   fn bottom() -> Self {
+      // unstable:
+      // Product(std::array::from_fn(|_| T::bottom()))
+      Product([(); N].map(|_| T::bottom()))
+   }
+
+   fn top() -> Self {
+      Product([(); N].map(|_| T::top()))
+   }
+}
+
+#[test]
+fn test_product_of_array_lattice() {
+   let a1 = Product([1, 5, 3]);
+   let a2 = Product([1, 3, 4]);
+   let a1_a2_meet = Product([1,3,3]);
+   let a1_a2_join = Product([1,5,4]);
+   assert_eq!(a1.meet(a2), a1_a2_meet);
+   assert_eq!(a1.join(a2), a1_a2_join);
+
+   assert_eq!(Product([0; 3]), Product::<[u32; 3]>::bottom());
+   assert_eq!(Product([true; 4]), Product::<[bool; 4]>::top());
+}
\ No newline at end of file
